"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _webpack = /*#__PURE__*/ _interopRequireWildcard(require("webpack"));
const _webpackManifestPlugin = require("webpack-manifest-plugin");
const _reactRefreshWebpackPlugin = /*#__PURE__*/ _interopRequireDefault(require("@pmmmwh/react-refresh-webpack-plugin"));
const _miniCssExtractPlugin = /*#__PURE__*/ _interopRequireDefault(require("mini-css-extract-plugin"));
const _manifest = /*#__PURE__*/ _interopRequireDefault(require("@/configs/manifest"));
const _mhy = /*#__PURE__*/ _interopRequireWildcard(require("@/configs/mhy"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if (typeof WeakMap !== "function") return null;
    var cacheBabelInterop = new WeakMap();
    var cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interopRequireWildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) {
        return obj;
    }
    if (obj === null || typeof obj !== "object" && typeof obj !== "function") {
        return {
            default: obj
        };
    }
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) {
        return cache.get(obj);
    }
    var newObj = {};
    var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj){
        if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) {
            var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
            if (desc && (desc.get || desc.set)) {
                Object.defineProperty(newObj, key, desc);
            } else {
                newObj[key] = obj[key];
            }
        }
    }
    newObj.default = obj;
    if (cache) {
        cache.set(obj, newObj);
    }
    return newObj;
}
// Remove default values, no use on client side
const getMhyConfig = ()=>{
    const config = JSON.parse(JSON.stringify(_mhy.default));
    for (const i of _mhy.defaultKeys){
        delete config[i];
    }
    return config;
};
var _default = (plugins = [])=>{
    _manifest.default.icons.map((icon)=>{
        const inCwdPath = _path.default.resolve(process.cwd(), icon.src);
        if (_fs.default.existsSync(inCwdPath)) {
            icon.src = inCwdPath;
        }
        return icon;
    });
    if (process.env.WEBPACK_DEV_SERVER) {
        plugins.push(new _webpack.HotModuleReplacementPlugin());
        plugins.push(new _reactRefreshWebpackPlugin.default());
    }
    return plugins.concat([
        // Disabled currently due to no WP5 support
        //new WebpackPwaManifestPlugin(manifest),
        new _webpackManifestPlugin.WebpackManifestPlugin({
            fileName: './manifest.webpack.json',
            generate: (seed, files, e)=>{
                const entries = JSON.parse(JSON.stringify(e));
                const output = {
                    __all: files.map(({ chunk , isAsset , isChunk , isInitial , isModuleAsset , name , path  })=>({
                            isAsset,
                            isChunk,
                            isInitial,
                            isModuleAsset,
                            name,
                            path
                        }))
                };
                for (const [key, chunks] of Object.entries(e)){
                    output[key] = chunks.map((entry)=>{
                        const { isAsset , isChunk , isInitial , isModuleAsset , name , path  } = files.find(({ path  })=>path.includes(entry));
                        return {
                            isAsset,
                            isChunk,
                            isInitial,
                            isModuleAsset,
                            name,
                            path
                        };
                    });
                }
                return output;
            }
        }),
        new _webpack.default.DefinePlugin({
            mhy: JSON.stringify(getMhyConfig()),
            'process.env': JSON.stringify({
                NODE_ENV: process.env.NODE_ENV
            })
        }),
        new _miniCssExtractPlugin.default({
            filename: process.env.NODE_ENV !== 'production' ? '[name].css' : '[name].[contenthash].css',
            chunkFilename: process.env.NODE_ENV !== 'production' ? '[id].css' : '[id].[contenthash].css',
            ignoreOrder: true
        })
    ]);
};
