"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _yargs = /*#__PURE__*/ _interopRequireDefault(require("yargs"));
const _formatSource = /*#__PURE__*/ _interopRequireDefault(require("@/utils/formatSource"));
const _fileTypes = /*#__PURE__*/ _interopRequireDefault(require("@/utils/fileTypes"));
const _figures = /*#__PURE__*/ _interopRequireDefault(require("figures"));
const _chalk = /*#__PURE__*/ _interopRequireDefault(require("chalk"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const commandHandler = (args)=>{
    const { config , init  } = args;
    if (!config.length || config.length > 1 || init // forced init
    ) {
        handleWrite(args);
    } else {
        handlePrint(args);
    }
};
const handleWrite = ({ config , dir , format , overwrite  })=>{
    const m = fetchWriters(config);
    const modules = m.sort((x, y)=>x.includes('typescript') ? -1 : y.includes('typescript') ? 1 : 0).map((module, i)=>{
        process.stdout.write(`\rLoading configs ${i + 1}/${m.length}`);
        return require(module);
    });
    const results = [];
    for (const writeConfig of modules){
        process.stdout.write(`\r Writing configs ${results.length + 1}/${modules.length}`);
        results.push(writeConfig(dir, format, overwrite));
    }
    try {
        process.stdout.clearLine();
        process.stdout.cursorTo(0);
    } catch  {}
    const existed = results.filter(({ isExisted  })=>isExisted);
    const overwritten = results.filter(({ isOverwritten  })=>isOverwritten);
    const cleanlyInitialized = results.filter(({ isExisted , isOverwritten  })=>!isOverwritten && !isExisted);
    if (cleanlyInitialized.length || overwritten.length) {
        console.log(`\nInitialized ${cleanlyInitialized.length + overwritten.length} config(s) successfully:\n`);
        for (const { filename , isOverwritten  } of [
            ...cleanlyInitialized,
            ...overwritten
        ]){
            console.log(`    ${_chalk.default.green(_figures.default.tick)} ${filename}${isOverwritten ? ` ${_chalk.default.yellow(`${_figures.default.warning} overwritten`)}` : ''}`);
        }
    }
    if (!overwrite && existed.length) {
        console.log('\n');
        process.stdout.write(`\rSkipped ${existed.length} already existing config(s):\n`);
        for (const { filename: filename1  } of existed){
            console.log(`    ${_chalk.default.blue(_figures.default.line)} ${filename1}`);
        }
        console.log('\nUse -o/--overwrite to force re-initialization of an already existing file.');
    }
};
const applyPath = (path, configObj)=>{
    if (!path) return configObj;
    return path.split('.').reduce((acc, p)=>acc[p], configObj);
};
const handlePrint = ({ config , format , path  })=>{
    const module = require(fetchConfigs(config)[0]);
    const configObj = applyPath(path, module.default || module);
    let formatted = (0, _formatSource.default)(configObj, format);
    if (format === _fileTypes.default.JSON) {
        formatted = require('json-colorizer')(formatted);
    }
    console.log(formatted);
};
const fetchConfigs = (config = [])=>{
    const glob = require('glob');
    const globStr = config.length ? `*(${config.join('|')})` : '*';
    return glob.sync(_path.default.resolve(__dirname, '../../configs/', globStr) + _path.default.sep);
};
const fetchWriters = (config = [])=>{
    const glob = require('glob');
    const globStr = config.length ? `*(${config.join('|')})/write.js` : '*/write.js';
    return glob.sync(_path.default.resolve(__dirname, '../../configs/', globStr));
};
var _default = ()=>{
    _yargs.default.command([
        'config',
        'c'
    ], 'write out to cwd all available configs (shorthand to --init)').command([
        'config [config...]',
        'c [config...]'
    ], 'init/print out specified configs only', (yargs)=>{
        yargs.positional('config', {
            describe: 'config file to load',
            type: 'string'
        }).option('dir', {
            alias: 'd',
            default: process.cwd(),
            description: 'Output directory'
        }).option('init', {
            alias: 'i',
            default: false,
            description: 'Initialize (create) config file',
            choices: [
                false,
                true
            ],
            type: 'boolean'
        }).option('overwrite', {
            alias: 'o',
            default: false,
            description: 'Overwrite existing files',
            choices: [
                false,
                true
            ],
            type: 'boolean'
        }).option('format', {
            alias: 'f',
            default: undefined,
            description: 'Output format',
            choices: Object.values(_fileTypes.default)
        }).option('path', {
            alias: 'p',
            default: undefined,
            description: 'You can narrow down the output to a given path. This only works while printing the output the console. Eg: loaders.0.use',
            type: 'string'
        });
    }, commandHandler);
};
