"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _yargs = /*#__PURE__*/ _interopRequireDefault(require("yargs"));
const _processes = require("@/processes");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const filterDeps = (d)=>!d.includes('eslint') && !d.includes('!');
const printDependencyList = (list, emptyMsg = 'No results!')=>{
    const longest = list.reduce((acc, [{ length  }])=>length > acc ? length : acc, 0);
    if (longest > 0) {
        console.log() // empty line
        ;
        list.sort(([a], [b])=>a > b ? 1 : -1).forEach(([name, version])=>console.log('  ', name.padEnd(longest + 3), version));
    } else {
        console.log(emptyMsg);
    }
};
const commandHandler = async ({ tool , prop  })=>{
    switch(tool){
        case 'collect':
            {
                const { dependencies: _dependencies  } = require(_path.default.resolve(__dirname, '../../../package.json'));
                const packageJSON = require(_path.default.resolve(process.cwd(), 'package.json'));
                const { dependencies ={}  } = packageJSON;
                const r = require('@/configs/mhy');
                const { defaultIgnoreList , buildFolder , distFolder  } = r.default || r;
                const options = {
                    ignoreDirs: [
                        ...defaultIgnoreList,
                        buildFolder,
                        distFolder
                    ]
                };
                const mhyDependencies = {};
                require('depcheck')(process.cwd(), options, async ({ missing , using  })=>{
                    const deps = new Set([
                        ...Object.keys(missing).filter(filterDeps),
                        ...Object.keys(using).filter(filterDeps)
                    ]);
                    for (const dep of deps){
                        if (!dependencies[dep] && _dependencies[dep]) {
                            mhyDependencies[dep] = _dependencies[dep];
                        }
                    }
                    printDependencyList(Object.entries(mhyDependencies));
                    if (prop === 'write') {
                        await require('write-pkg')(process.cwd(), {
                            ...packageJSON,
                            mhyDependencies
                        });
                    } else if (prop === 'write-peer') {
                        await require('write-pkg')(process.cwd(), {
                            ...packageJSON,
                            peerDependencies: mhyDependencies
                        });
                    } else if (prop === 'write-both') {
                        await require('write-pkg')(process.cwd(), {
                            ...packageJSON,
                            peerDependencies: mhyDependencies,
                            mhyDependencies
                        });
                    }
                    prop.includes('write') && console.log('\n   Dependencies have been written into package.json successfully!');
                });
                break;
            }
        case 'install':
            {
                const Install = (0, _processes.loadProcess)('install')();
                new Install(prop);
                break;
            }
        case 'search':
        default:
            {
                const all = require(_path.default.resolve(__dirname, '../../../package.json')).dependencies;
                const entries = Object.entries(all);
                const filtered = entries.filter(([name])=>!prop.length || name.includes(prop));
                printDependencyList(filtered);
            }
    }
};
var _default = ()=>{
    _yargs.default.command([
        'dep [tool] [prop]',
        'deps [tool] [prop]'
    ], 'dependency management tool', (y)=>{
        y.positional('tool', {
            describe: 'Which task of the tool you want to run?',
            type: 'string',
            default: 'search',
            choices: [
                'search',
                'collect',
                'install'
            ]
        }).positional('prop', {
            describe: `search [search-string: ''] | collect [''|write|write-peer|write-both]`,
            type: 'string',
            default: ''
        });
    }, commandHandler);
};
