"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _fsExtra = /*#__PURE__*/ _interopRequireDefault(require("fs-extra"));
const _copyDir = /*#__PURE__*/ _interopRequireDefault(require("copy-dir"));
const _fileTypes = /*#__PURE__*/ _interopRequireDefault(require("@/utils/fileTypes"));
const _processes = /*#__PURE__*/ _interopRequireDefault(require("@/processes"));
const _mhy = /*#__PURE__*/ _interopRequireDefault(require("@/configs/mhy"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const SWCPath = _path.default.resolve(__dirname, '.swcrc');
const getCmdSWCCLI = (flags = [])=>[
        'node',
        require.resolve('@swc/cli'),
        _path.default.resolve(process.cwd(), _mhy.default.srcFolder),
        '--out-dir',
        _mhy.default.distFolder,
        '--config-file',
        SWCPath,
        '--ignore',
        [
            'node_modules',
            'test',
            'tests',
            'temp',
            'tmp',
            '**/*.d.ts',
            _mhy.default.distFolder,
            _mhy.default.buildFolder
        ].join(','),
        ...flags
    ];
class SWC extends _processes.default {
    processLine(line) {
        if (line && !line.includes('could not replace') && !line.includes('tscpaths') && !line.includes('Replaced 0 paths')) {
            return line;
        }
    }
    constructor(args){
        try {
            _fsExtra.default.unlinkSync(_path.default.resolve(process.cwd(), _mhy.default.distFolder));
        } catch  {}
        const fn = require('@/configs/swc/write');
        (fn.default || fn)(__dirname, _fileTypes.default.JSON_NO_EXT, true);
        const { props: { defaultAction ='start'  } = {} , ...rest } = args;
        super(args);
        this.onStart = ({ name  }, { flags  })=>{
            this.spawn(name, getCmdSWCCLI(flags), 'pipe', false);
            this.on('data', (line)=>{
                if (!line.includes('Successfully')) return;
                _copyDir.default.sync(_path.default.resolve(process.cwd(), _mhy.default.srcFolder), _path.default.resolve(process.cwd(), _mhy.default.distFolder), {
                    cover: false,
                    mode: true,
                    filter: function(stat, filepath) {
                        if (stat === 'file') {
                            if (filepath.endsWith('.d.ts')) {
                                return true;
                            }
                            if (filepath.endsWith('ts') || filepath.endsWith('tsx') || filepath.endsWith('js') || filepath.endsWith('jsx')) {
                                return false;
                            }
                            return true;
                        }
                        return true;
                    }
                });
                const configPaths = [
                    _path.default.resolve(process.cwd(), 'tsconfig.json'),
                    _path.default.resolve(process.cwd(), 'jsconfig.json')
                ];
                const configPath = configPaths.find((p)=>_fsExtra.default.existsSync(p));
                if (!configPath) return;
                // Fix tsc paths
                const p = this.spawn('fix_tsc_path', [
                    'node',
                    require.resolve('tscpaths/cjs'),
                    '-p',
                    configPath,
                    '-s',
                    _mhy.default.srcFolder,
                    '-o',
                    _mhy.default.distFolder
                ], [
                    'pipe',
                    'pipe',
                    'pipe'
                ], false);
            });
        };
        this.actions = [
            {
                name: 'start',
                enabled: true,
                onRun: this.onStart
            }
        ];
        this.run(defaultAction, {
            ...rest
        });
    }
}
const getSWC = ()=>SWC;
var _default = getSWC;
