"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _processes = /*#__PURE__*/ _interopRequireDefault(require("@/processes"));
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _mhy = /*#__PURE__*/ _interopRequireDefault(require("@/configs/mhy"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const getCmdWebpackCLI = ()=>[
        'node',
        require.resolve('webpack-cli/bin/cli'),
        '--config',
        require.resolve('@/configs/webpack'),
        '--profile',
        '--json'
    ];
const getCmdWebpackBundleAnalyzerCLI = (flags)=>[
        'node',
        require.resolve('webpack-bundle-analyzer/lib/bin/analyzer'),
        ...flags
    ];
class WebpackBundleAnalyzer extends _processes.default {
    constructor(args){
        const { props: { defaultAction ='generateStats'  } = {} , ...rest } = args;
        super(args);
        this.onGenerateStats = ({ name  }, { flags =[]  })=>{
            console.log('Gathering bundle stats');
            const p = this.spawn(name, getCmdWebpackCLI(flags), [
                'pipe'
            ], false);
            const jsonPath = _path.default.resolve(__dirname, 'stats.json');
            const bundlePath = _path.default.resolve(process.cwd(), _mhy.default.buildFolder);
            let json = '';
            let all = '';
            let started = false;
            this.on('data', (d)=>{
                all = all + d;
                if (!started && d.includes('{\n')) {
                    const [, ...trimmed] = d.split('{\n');
                    d = `{${trimmed.join('{\n')}`;
                    started = true;
                }
                if (started) {
                    json += d;
                }
            });
            p.on('exit', ()=>{
                if (json.length === 0) {
                    console.log(all);
                    return;
                }
                _fs.default.writeFileSync(jsonPath, json);
                _fs.default.writeFileSync(jsonPath + '.out.txt', all);
                console.log('Stats gathered successfully, analyzing...');
                this.run('start', {
                    ...this.rest,
                    flags: [
                        ...flags,
                        jsonPath,
                        bundlePath
                    ]
                });
            });
        };
        this.onStart = ({ name  }, { flags =[]  })=>this.spawn(name, getCmdWebpackBundleAnalyzerCLI(flags));
        this.actions = [
            {
                name: 'start',
                enabled: true,
                onRun: this.onStart
            },
            {
                name: 'generateStats',
                enabled: true,
                onRun: this.onGenerateStats
            }
        ];
        this.rest = rest;
        this.run(defaultAction, {
            ...rest
        });
    }
}
var _default = ()=>WebpackBundleAnalyzer;
